package gregapi.block.multitileentity;

import gregapi.code.ArrayListNoNulls;
import gregapi.oredict.OreDictMaterialStack;

import java.util.List;
import java.util.Random;

import net.minecraft.block.Block;
import net.minecraft.client.particle.EffectRenderer;
import net.minecraft.client.renderer.texture.IIconRegister;
import net.minecraft.entity.Entity;
import net.minecraft.entity.EntityLivingBase;
import net.minecraft.entity.EnumCreatureType;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.inventory.IInventory;
import net.minecraft.item.ItemStack;
import net.minecraft.util.AxisAlignedBB;
import net.minecraft.util.ChunkCoordinates;
import net.minecraft.util.MovingObjectPosition;
import net.minecraft.util.Vec3;
import net.minecraft.world.Explosion;
import net.minecraft.world.World;
import net.minecraftforge.common.IPlantable;
import net.minecraftforge.common.util.ForgeDirection;
import cpw.mods.fml.relauncher.Side;
import cpw.mods.fml.relauncher.SideOnly;

/**
 * @author Gregorius Techneticies
 * 
 * Interfaces to override Block Functionality.
 */
public class MultiTileEntityInterfaces {
	// Hooks into the Block Class.
    public static interface IMultiTileEntityOnBlockExploded					{public void onExploded(Explosion aExplosion);}
    public static interface IMultiTileEntityGetPickBlock					{public ItemStack getPickBlock(MovingObjectPosition aTarget);}
    public static interface IMultiTileEntityBreakBlock						{public void breakBlock();}
    public static interface IMultiTileEntityGetStackFromBlock				{public ItemStack getStackFromBlock(byte aSide);}
    public static interface IMultiTileEntityGetFlammability					{public int getFlammability(byte aSide);}
    public static interface IMultiTileEntityGetFireSpreadSpeed				{public int getFireSpreadSpeed(byte aSide);}
    public static interface IMultiTileEntityIsFireSource					{public boolean isFireSource(byte aSide);}
	public static interface IMultiTileEntityCanEntityDestroy				{public boolean canEntityDestroy(Entity aEntity);}
	public static interface IMultiTileEntityOnToolClick						{public long onToolClick(String aTool, long aRemainingDurability, Entity aPlayer, IInventory aPlayerInventory, boolean aSneaking, ItemStack aStack, byte aSide, float aHitX, float aHitY, float aHitZ);}
	public static interface IMultiTileEntityGetMaterialAtSide				{public OreDictMaterialStack getMaterialAtSide(byte aSide);}
	public static interface IMultiTileEntityRemoveMaterialFromSide			{public boolean removeMaterialFromSide(byte aSide, OreDictMaterialStack aMaterial);}
	public static interface IMultiTileEntityGetDrops						{public ArrayListNoNulls<ItemStack> getDrops(int aFortune, boolean aSilkTouch);}
    public static interface IMultiTileEntityGetBlockHardness				{public float getBlockHardness(EntityPlayer aPlayer);}
    public static interface IMultiTileEntityGetExplosionResistance			{public float getExplosionResistance(Entity aExploder, double aExplosionX, double aExplosionY, double aExplosionZ);}
    public static interface IMultiTileEntityIsSideSolid						{public boolean isSideSolid(byte aSide);}
    public static interface IMultiTileEntityIsBeaconBase					{public boolean isBeaconBase(int aBeaconX, int aBeaconY, int aBeaconZ);}
    public static interface IMultiTileEntityGetLightOpacity					{public int getLightOpacity();}
	public static interface IMultiTileEntityGetBlocksMovement				{public boolean getBlocksMovement();}
	public static interface IMultiTileEntityGetMixedBrightnessForBlock		{public int getMixedBrightnessForBlock();}
	public static interface IMultiTileEntityShouldSideBeRendered			{public boolean shouldSideBeRendered(byte aSide);}
	public static interface IMultiTileEntityIsBlockSolid					{public boolean isBlockSolid(byte aSide);}
	public static interface IMultiTileEntityAddCollisionBoxesToList			{public void addCollisionBoxesToList(AxisAlignedBB aAABB, List aList, Entity aEntity);}
	public static interface IMultiTileEntityGetCollisionBoundingBoxFromPool	{public AxisAlignedBB getCollisionBoundingBoxFromPool();}
	public static interface IMultiTileEntityGetSelectedBoundingBoxFromPool	{public AxisAlignedBB getSelectedBoundingBoxFromPool();}
	public static interface IMultiTileEntityUpdateTick						{public void updateTick(Random aRandom);}
	public static interface IMultiTileEntityRandomDisplayTick				{public void randomDisplayTick(Random aRandom);}
	public static interface IMultiTileEntityOnBlockDestroyedByPlayer		{public void onBlockDestroyedByPlayer(int aRandom);}
	public static interface IMultiTileEntityOnBlockAdded					{public void onBlockAdded();}
	public static interface IMultiTileEntityDropXpOnBlockBreak				{public void dropXpOnBlockBreak(int aXP);}
	public static interface IMultiTileEntityCollisionRayTrace				{public MovingObjectPosition collisionRayTrace(Vec3 aVectorA, Vec3 aVectorB);}
	public static interface IMultiTileEntityOnBlockDestroyedByExplosion		{public void onBlockDestroyedByExplosion(Explosion aExplosion);}
	public static interface IMultiTileEntityOnBlockActivated				{public boolean onBlockActivated(EntityPlayer aPlayer, byte aSide, float aHitX, float aHitY, float aHitZ);}
	public static interface IMultiTileEntityOnEntityWalking					{public void onEntityWalking(Entity aEntity);}
	public static interface IMultiTileEntityOnBlockClicked					{public void onBlockClicked(EntityPlayer aPlayer);}
	public static interface IMultiTileEntityVelocityToAddToEntity			{public void velocityToAddToEntity(Entity aEntity, Vec3 aVector);}
	public static interface IMultiTileEntitySetBlockBoundsBasedOnState		{public void setBlockBoundsBasedOnState();}
	public static interface IMultiTileEntityIsProvidingWeakPower			{public int isProvidingWeakPower(byte aSide);}
	public static interface IMultiTileEntityIsProvidingStrongPower			{public int isProvidingStrongPower(byte aSide);}
	public static interface IMultiTileEntityOnEntityCollidedWithBlock		{public void onEntityCollidedWithBlock(Entity aEntity);}
	public static interface IMultiTileEntityCanBlockStay					{public boolean canBlockStay();}
	public static interface IMultiTileEntityOnFallenUpon					{public void onFallenUpon(Entity aEntity, float aFallDistance);}
	public static interface IMultiTileEntityOnBlockHarvested				{public void onBlockHarvested(int aMetaData, EntityPlayer aPlayer);}
	public static interface IMultiTileEntityOnBlockPreDestroy				{public void onBlockPreDestroy(int aMetaData);}
	public static interface IMultiTileEntityFillWithRain					{public void fillWithRain();}
	public static interface IMultiTileEntityGetComparatorInputOverride		{public int getComparatorInputOverride(byte aSide);}
	public static interface IMultiTileEntityGetLightValue					{public int getLightValue();}
	public static interface IMultiTileEntityIsLadder						{public boolean isLadder(EntityLivingBase aEntity);}
	public static interface IMultiTileEntityIsNormalCube					{public boolean isNormalCube();}
	public static interface IMultiTileEntityIsReplaceable					{public boolean isReplaceable();}
	public static interface IMultiTileEntityIsBurning						{public boolean isBurning();}
	public static interface IMultiTileEntityIsAir							{public boolean isAir();}
	public static interface IMultiTileEntityRemovedByPlayer					{public boolean removedByPlayer(World aWorld, EntityPlayer aPlayer, boolean aWillHarvest);}
	public static interface IMultiTileEntityCanCreatureSpawn				{public boolean canCreatureSpawn(EnumCreatureType aType);}
	public static interface IMultiTileEntityIsBed							{public boolean isBed(EntityLivingBase aPlayer);}
	public static interface IMultiTileEntityGetBedSpawnPosition				{public ChunkCoordinates getBedSpawnPosition(EntityPlayer aPlayer);}
	public static interface IMultiTileEntitySetBedOccupied					{public void setBedOccupied(EntityPlayer aPlayer, boolean aOccupied);}
	public static interface IMultiTileEntityGetBedDirection					{public int getBedDirection();}
	public static interface IMultiTileEntityIsBedFoot						{public boolean isBedFoot();}
	public static interface IMultiTileEntityBeginLeavesDecay				{public void beginLeavesDecay();}
	public static interface IMultiTileEntityCanSustainLeaves				{public boolean canSustainLeaves();}
	public static interface IMultiTileEntityIsLeaves						{public boolean isLeaves();}
	public static interface IMultiTileEntityCanBeReplacedByLeaves			{public boolean canBeReplacedByLeaves();}
	public static interface IMultiTileEntityIsWood							{public boolean isWood();}
	public static interface IMultiTileEntityIsReplaceableOreGen				{public boolean isReplaceableOreGen(Block aTarget);}
	public static interface IMultiTileEntityCanConnectRedstone				{public boolean canConnectRedstone(byte aSide);}
	public static interface IMultiTileEntityCanPlaceTorchOnTop				{public boolean canPlaceTorchOnTop();}
	public static interface IMultiTileEntityIsFoliage						{public boolean isFoliage();}
	public static interface IMultiTileEntityCanSustainPlant					{public boolean canSustainPlant(byte aSide, IPlantable aPlantable);}
	public static interface IMultiTileEntityOnPlantGrow						{public void onPlantGrow(int sX, int sY, int sZ);}
	public static interface IMultiTileEntityIsFertile						{public boolean isFertile();}
	public static interface IMultiTileEntityRotateBlock						{public boolean rotateBlock(byte aSide);}
	public static interface IMultiTileEntityGetValidRotations				{public ForgeDirection[] getValidRotations();}
	public static interface IMultiTileEntityGetEnchantPowerBonus			{public float getEnchantPowerBonus();}
	public static interface IMultiTileEntityRecolourBlock					{public boolean recolourBlock(byte aSide, byte aColor);}
	public static interface IMultiTileEntityShouldCheckWeakPower			{public boolean shouldCheckWeakPower(byte aSide);}
	public static interface IMultiTileEntityGetWeakChanges					{public boolean getWeakChanges();}
	public static interface IMultiTileEntityRegisterIcons					{@SideOnly(Side.CLIENT) public void registerIcons(IIconRegister aIconRegister);}
	public static interface IMultiTileEntityAddHitEffects					{@SideOnly(Side.CLIENT) public boolean addHitEffects(World aWorld, MovingObjectPosition aTarget, EffectRenderer aRenderer);}
	public static interface IMultiTileEntityAddDestroyEffects				{@SideOnly(Side.CLIENT) public boolean addDestroyEffects(int aMetaData, EffectRenderer aRenderer);}
	
	public static interface IMultiTileEntityAddToolTips {
		/** Adds ToolTips to the Item. */
		public void addToolTips(List aList, ItemStack aStack, boolean aF3_H);
	}
	
	public static interface IMultiTileEntityOnRegistration {
		/** Called when the TileEntity is being registered. */
		public void onRegistration();
	}
	
	public static interface IMultiTileEntityCanPlace {
		/** Return false if this TileEntity cannot be placed at that Location. */
		public boolean canPlace(ItemStack aStack, EntityPlayer aPlayer, World aWorld, int aX, int aY, int aZ, byte aSide, float aHitX, float aHitY, float aHitZ);
	}
	
	public static interface IMultiTileEntityOnPlaced {
		/** Return false to prevent the Sound from being played, when the Block is placed. */
		public boolean onPlaced(ItemStack aStack, EntityPlayer aPlayer, World aWorld, int aX, int aY, int aZ, byte aSide, float aHitX, float aHitY, float aHitZ);
	}
}